@@ -1,71 +1,39 @@
 # -*- coding: utf-8 -*-
 """Page Admin module."""
 from pages import settings
-from pages.models import Page, Content, PageAlias
+from pages.models import Page, Content, PageAlias, Media
 from pages.phttp import get_language_from_request, get_template_from_request
 from pages.utils import get_placeholders
 from pages.templatetags.pages_tags import PlaceholderNode
 from pages.admin.forms import make_form
-from pages.admin.views import traduction, get_content, sub_menu
-from pages.admin.views import list_pages_ajax
-from pages.admin.views import change_status, modify_content, delete_content
-from pages.admin.views import move_page
-from pages.permissions import PagePermission
-import pages.widgets
+from pages.admin import views
+from pages.urlconf_registry import registry
 
+from os.path import join
+import urllib
+from collections import defaultdict
 from django.contrib import admin
-from django.utils.translation import ugettext as _, ugettext_lazy
+from django.utils.translation import ugettext_lazy as _
 from django.utils.encoding import force_text
 from django.conf import settings as global_settings
 from django.http import HttpResponseRedirect, Http404
-from django.contrib.admin.util import unquote
 from django.contrib.admin.sites import AlreadyRegistered
-if global_settings.USE_I18N:
-    from django.views.i18n import javascript_catalog
-else:
-    from django.views.i18n import null_javascript_catalog as javascript_catalog
 
-from os.path import join
-
-
-from django.db import models
-def create_page_model(placeholders=None):
-    """
-    Create Page model
-    """
-    if placeholders is None:
-        placeholders = []
-    app_label='pages'
-    module = 'pages.models.test'
-    class Meta:
-        # Using type('Meta', ...) gives a dictproxy error during model creation
-        pass
-
-    # app_label must be set using the Meta inner class
-    setattr(Meta, 'app_label', app_label)
-
-    # Set up a dictionary to simulate declarations within a class
-    attrs = {'__module__': module, 'Meta': Meta}
-
-    # Add in any fields that were provided
-    for p in placeholders:
-        attrs[p.ctype] = models.TextField(blank=True)
-
-    attrs["slug"] = models.TextField()
-    attrs["title"] = models.TextField()
-
-    # Create the class, which automatically triggers ModelBase processing
-    model = type("Page", (Page,), attrs)
-
-    return model
+extra_actions_registery = []
+def add_page_action(action):
+    if action not in extra_actions_registery:
+        extra_actions_registery.append(action)
 
 
 class PageAdmin(admin.ModelAdmin):
     """Page Admin class."""
 
+    actions = extra_actions_registery
+
     # these mandatory fields are not versioned
     mandatory_placeholders = ('title', 'slug')
-    general_fields = ['title', 'slug', 'status', 'target',
+    general_fields = [
+        'title', 'slug', 'status', 'target',
         'position', 'freeze_date', 'template', 'language',
         'redirect_to', 'redirect_to_url']
 
@@ -86,10 +54,8 @@ class PageAdmin(admin.ModelAdmin):
     if settings.PAGE_SHOW_START_DATE:
         general_fields.insert(insert_point, 'publication_date')
 
-    from pages.urlconf_registry import registry
-    if(len(registry)):
-        general_fields.append('delegate_to')
-        insert_point = general_fields.index('status') + 1
+    general_fields.append('delegate_to')
+    insert_point = general_fields.index('status') + 1
 
     page_templates = settings.get_page_templates()
 
@@ -102,61 +68,50 @@ class PageAdmin(admin.ModelAdmin):
 
     class Media:
         css = {
-            'all': [join(settings.PAGES_MEDIA_URL, path) for path in (
+            'all': [join(settings.PAGES_STATIC_URL, path) for path in (
                 'css/rte.css',
                 'css/pages.css'
             )]
         }
-        js = [join(settings.PAGES_MEDIA_URL, path) for path in (
+        js = [join(settings.PAGES_STATIC_URL, path) for path in (
             'javascript/jquery.js',
             'javascript/jquery.rte.js',
             'javascript/pages.js',
             'javascript/pages_list.js',
             'javascript/pages_form.js',
             'javascript/jquery.query-2.1.7.js',
+            'javascript/iframeResizer.min.js',
         )]
 
-    @classmethod
-    def add_action(cls, method):
-        if method not in cls.actions:
-            cls.actions.append(method)
+    def get_urls(self):
+        urls = super(PageAdmin, self).get_urls()
+        from django.conf.urls import url
 
-    def urls(self):
-        from django.conf.urls import patterns, url, include
-
-        # Admin-site-wide views.
-        urlpatterns = patterns('',
-            url(r'^$', self.list_pages, name='page-index'),
+        pages_urls = [
+            url(r'^$', self.list_pages, name='page-changelist'),
             url(r'^(?P<page_id>[0-9]+)/traduction/(?P<language_id>[-\w]+)/$',
-                traduction, name='page-traduction'),
+                views.traduction, name='page-traduction'),
             url(r'^(?P<page_id>[0-9]+)/get-content/(?P<content_id>[0-9]+)/$',
-                get_content, name='page-get-content'),
+                views.get_content, name='page-get-content'),
             url(r'^(?P<page_id>[0-9]+)/modify-content/(?P<content_type>[-\w]+)/(?P<language_id>[-\w]+)/$',
-                modify_content, name='page-modify-content'),
+                views.modify_content, name='page-modify-content'),
+            url(r'^(?P<page_id>[0-9]+)/modify-placeholder/$',
+                views.modify_placeholder, name='page-modify-placeholder'),
+            url(r'^(?P<page_id>[0-9]+)/get-last-content/$',
+                views.get_last_content, name='page-get-last-content'),
             url(r'^(?P<page_id>[0-9]+)/delete-content/(?P<language_id>[-\w]+)/$',
-                delete_content, name='page-delete-content'),
+                views.delete_content, name='page-delete-content'),
             url(r'^(?P<page_id>[0-9]+)/sub-menu/$',
-                sub_menu, name='page-sub-menu'),
+                views.sub_menu, name='page-sub-menu'),
             url(r'^(?P<page_id>[0-9]+)/move-page/$',
-                move_page, name='page-move-page'),
+                views.move_page, name='page-move-page'),
             url(r'^(?P<page_id>[0-9]+)/change-status/$',
-                change_status, name='page-change-status'),
-        )
+                views.change_status, name='page-change-status'),
+            url(r'^(?P<media_id>[0-9]+)/media-url/$',
+                views.get_media_url, name='get-media-url'),
+        ]
 
-        urlpatterns += super(PageAdmin, self).urls
-
-        return urlpatterns
-
-    urls = property(urls)
-
-    def i18n_javascript(self, request):
-        """Displays the i18n JavaScript that the Django admin
-        requires.
-
-        This takes into account the ``USE_I18N`` setting. If it's set to False, the
-        generated JavaScript will be leaner and faster.
-        """
-        return javascript_catalog(request, packages='pages')
+        return pages_urls + urls
 
     def save_model(self, request, page, form, change):
         """Move the page in the tree if necessary and save every
@@ -199,8 +154,6 @@ class PageAdmin(admin.ModelAdmin):
         Add fieldsets of placeholders to the list of already
         existing fieldsets.
         """
-        general_fields = list(self.general_fields)
-        perms = PagePermission(request.user)
 
         # some ugly business to remove freeze_date
         # from the field list
@@ -210,25 +163,34 @@ class PageAdmin(admin.ModelAdmin):
         }
 
         default_fieldsets = list(self.fieldsets)
-        if not perms.check('freeze'):
+        if not request.user.has_perm('pages.can_freeze'):
             general_module['fields'].remove('freeze_date')
-        if not perms.check('publish'):
+        if not request.user.has_perm('pages.can_publish'):
             general_module['fields'].remove('status')
+        if not len(registry):
+            general_module['fields'].remove('delegate_to')
 
         default_fieldsets[0][1] = general_module
 
         placeholder_fieldsets = []
+        section_placeholder_fieldsets = defaultdict(list)
         template = get_template_from_request(request, obj)
         for placeholder in get_placeholders(template):
-            if placeholder.ctype not in self.mandatory_placeholders:
+            if placeholder.ctype not in self.mandatory_placeholders and not placeholder.section:
                 placeholder_fieldsets.append(placeholder.ctype)
+            elif placeholder.section:
+                section_placeholder_fieldsets[placeholder.section].append(placeholder.ctype)
 
         additional_fieldsets = []
+        for title, fieldset in section_placeholder_fieldsets.items():
+            additional_fieldsets.append((title, {
+                'fields': fieldset,
+                'classes': ('module-content collapse grp-collapse grp-closed',),
+            }))
         additional_fieldsets.append((_('Content'), {
             'fields': placeholder_fieldsets,
             'classes': ('module-content',),
         }))
-
         return default_fieldsets + additional_fieldsets
 
     def save_form(self, request, form, change):
@@ -247,9 +209,9 @@ class PageAdmin(admin.ModelAdmin):
 
         template = get_template_from_request(request, obj)
 
-        model = create_page_model(get_placeholders(template))
+        #model = create_page_model(get_placeholders(template))
 
-        form = make_form(model)
+        form = make_form(self.model, get_placeholders(template))
 
         # bound the form
         language = get_language_from_request(request)
@@ -262,17 +224,18 @@ class PageAdmin(admin.ModelAdmin):
 
         template = get_template_from_request(request, obj)
         page_templates = settings.get_page_templates()
-        if len(page_templates) > 0:
-            template_choices = list(page_templates)
+        template_choices = list(page_templates)
+        # is default template is not in the list add it
+        if not [tpl for tpl in template_choices if tpl[0] == settings.PAGE_DEFAULT_TEMPLATE]:
             template_choices.insert(0, (settings.PAGE_DEFAULT_TEMPLATE,
                     _('Default template')))
-            form.base_fields['template'].choices = template_choices
-            form.base_fields['template'].initial = force_text(template)
+        form.base_fields['template'].choices = template_choices
+        form.base_fields['template'].initial = force_text(template)
 
         for placeholder in get_placeholders(template):
             ctype = placeholder.ctype
             if obj:
-                initial = placeholder.get_content(obj, language, ctype)
+                initial = placeholder.get_content(obj, language, lang_fallback=False)
             else:
                 initial = None
             form.base_fields[ctype] = placeholder.get_field(obj,
@@ -316,7 +279,6 @@ class PageAdmin(admin.ModelAdmin):
             else:
                 addlanguage = False
             if addlanguage:
-                from six.moves import urllib
                 splitted = list(urllib.parse.urlparse(response.url))
                 query = urllib.parse.parse_qs(splitted[4])
                 query['language'] = language
@@ -324,7 +286,6 @@ class PageAdmin(admin.ModelAdmin):
                 response = HttpResponseRedirect(urllib.parse.urlunparse(splitted))
         return response
 
-
     def add_view(self, request, form_url='', extra_context=None):
         """The ``add`` admin view for the :class:`Page <pages.models.Page>`."""
         extra_context = {
@@ -332,26 +293,21 @@ class PageAdmin(admin.ModelAdmin):
             'page_languages': settings.PAGE_LANGUAGES,
         }
         return super(PageAdmin, self).add_view(request, form_url,
-                                                            extra_context)
+                                               extra_context)
 
     def has_add_permission(self, request):
         """Return ``True`` if the current user has permission to add a new
         page."""
-        lang = get_language_from_request(request)
-        return PagePermission(request.user).check('add', lang=lang)
+        return request.user.has_perm('pages.add_page')
 
     def has_change_permission(self, request, obj=None):
         """Return ``True`` if the current user has permission
         to change the page."""
-        lang = get_language_from_request(request)
-        return PagePermission(request.user).check('change', page=obj,
-            lang=lang, method=request.method)
+        return request.user.has_perm('pages.change_page')
 
     def has_delete_permission(self, request, obj=None):
         """Return ``True`` if the current user has permission on the page."""
-        lang = get_language_from_request(request)
-        return PagePermission(request.user).check('change', page=obj,
-            lang=lang)
+        return request.user.has_perm('pages.delete_page')
 
     def list_pages(self, request, template_name=None, extra_context=None):
         """List root pages"""
@@ -370,12 +326,10 @@ class PageAdmin(admin.ModelAdmin):
         if settings.PAGE_HIDE_SITES:
             pages = pages.filter(sites=global_settings.SITE_ID)
 
-        perms = PagePermission(request.user)
         context = {
-            'can_publish': perms.check('publish'),
+            'can_publish': request.user.has_perm('pages.can_publish'),
             'can_import': settings.PAGE_IMPORT_ENABLED,
-            'language': language,
-            'name': _("page"),
+            'lang': language,  # lang is the key used by show_content
             'pages': pages,
             'opts': self.model._meta,
             'q': query
@@ -392,20 +346,28 @@ try:
 except AlreadyRegistered:
     pass
 
-
 class ContentAdmin(admin.ModelAdmin):
     list_display = ('__unicode__', 'type', 'language', 'page')
     list_filter = ('page',)
     search_fields = ('body',)
 
-#admin.site.register(Content, ContentAdmin)
 
 class AliasAdmin(admin.ModelAdmin):
     list_display = ('page', 'url',)
     list_editable = ('url',)
+    raw_id_fields = ['page']
 
 try:
     admin.site.register(PageAlias, AliasAdmin)
 except AlreadyRegistered:
     pass
 
+
+class MediaAdmin(admin.ModelAdmin):
+    list_display = ('image', 'title', 'creation_date', 'description', 'extension')
+    list_display_links = ('image', 'title', )
+
+try:
+    admin.site.register(Media, MediaAdmin)
+except AlreadyRegistered:
+    pass
\ No newline at end of file
