@@ -1,23 +1,28 @@
 # -*- coding: utf-8 -*-
 """Pages admin views"""
 from pages import settings
-from pages.models import Page, Content
+from pages.models import Page, Content, Media
 from pages.utils import get_placeholders
 from pages.phttp import get_language_from_request
-from pages.permissions import PagePermission
 
-from django.shortcuts import get_object_or_404, render_to_response
-from django.template import RequestContext
+from django.shortcuts import get_object_or_404, render
 from django.http import HttpResponse, Http404, HttpResponseRedirect
 from django.contrib.admin.views.decorators import staff_member_required
+from django.template import RequestContext
+from django import template
 from django.views.decorators.csrf import csrf_exempt
+from django.urls import reverse
+from django import forms
+from pages.managers import fake_page
 
+
+@staff_member_required
 @csrf_exempt
 def change_status(request, page_id):
     """
     Switch the status of a page.
     """
-    perm = PagePermission(request.user).check('change', method='POST')
+    perm = request.user.has_perm('pages.change_page')
     if perm and request.method == 'POST':
         page = Page.objects.get(pk=page_id)
         page.status = int(request.POST['status'])
@@ -25,65 +30,136 @@ def change_status(request, page_id):
         page.save()
         return HttpResponse(str(page.status))
     raise Http404
-change_status = staff_member_required(change_status)
 
+
+@staff_member_required
 def list_pages_ajax(request, invalid_move=False):
     """Render pages table for ajax function."""
     language = get_language_from_request(request)
     pages = Page.objects.root()
-    perms = PagePermission(request.user)
     context = {
-        'can_publish': perms.check('publish'),
-        'invalid_move':invalid_move,
+        'can_publish': request.user.has_perm('pages.can_publish'),
+        'invalid_move': invalid_move,
         'language': language,
         'pages': pages,
     }
-    return render_to_response("admin/pages/page/change_list_table.html", 
-        context,
-        context_instance=RequestContext(request))
-list_pages_ajax = staff_member_required(list_pages_ajax)
+    return render(
+        request, "admin/pages/page/change_list_table.html",
+        context)
 
+
+@staff_member_required
 @csrf_exempt
 def modify_content(request, page_id, content_type, language_id):
     """Modify the content of a page."""
     page = get_object_or_404(Page, pk=page_id)
-    perm = PagePermission(request.user).check('change', page=page,
-            lang=language_id, method='POST')
+    perm = request.user.has_perm('pages.change_page')
     if perm and request.method == 'POST':
         content = request.POST.get('content', False)
         if not content:
             raise Http404
-        page = Page.objects.get(pk=page_id)
         if settings.PAGE_CONTENT_REVISION:
             Content.objects.create_content_if_changed(page, language_id,
                                                       content_type, content)
         else:
             Content.objects.set_or_create_content(page, language_id,
                                                   content_type, content)
-        page.invalidate()
-        # to update last modification date
-        page.save()
+        if page:
+            page.invalidate()
+            # to update last modification date
+            page.save()
+        else:
+            fake_page.invalidate(content_type)
 
         return HttpResponse('ok')
     raise Http404
-modify_content = staff_member_required(modify_content)
 
+
+@staff_member_required
+@csrf_exempt
+def modify_placeholder(request, page_id):
+    """Modify the content of a page."""
+    content_type = request.GET.get('content_type')
+    language_id = request.GET.get('language_id')
+    page = get_object_or_404(Page, pk=page_id)
+    perm = request.user.has_perm('pages.change_page')
+    if perm and request.method == 'POST':
+        placeholders = get_placeholders(page.get_template())
+        for placeholder in placeholders:
+            if placeholder.name == content_type:
+                initial = placeholder.get_content(
+                    page, language_id, lang_fallback=False)
+                form = forms.Form(request.POST, request.FILES)
+                form.fields[content_type] = placeholder.get_field(
+                    page, language_id, initial=initial)
+                if not form.is_valid():
+                    return HttpResponse(form.as_p())
+
+                if placeholder.shared:
+                    save_page = None
+                else:
+                    save_page = page
+
+                extra_data = placeholder.get_extra_data(form.data)
+
+                placeholder.save(
+                    save_page, language_id,
+                    form.cleaned_data[content_type], True, extra_data)
+                if save_page:
+                    page.invalidate()
+                    # to update last modification date
+                    page.save()
+                else:
+                    fake_page.invalidate(content_type)
+
+                return HttpResponse('ok')
+        raise Http404("Content type not found in placeholders")
+
+    raise Http404
+
+
+@staff_member_required
+def get_last_content(request, page_id):
+    """Get the latest content for a particular type"""
+    content_type = request.GET.get('content_type')
+    language_id = request.GET.get('language_id')
+    page = get_object_or_404(Page, pk=page_id)
+    placeholders = get_placeholders(page.get_template())
+    _template = template.loader.get_template(page.get_template())
+    for placeholder in placeholders:
+        if placeholder.name == content_type:
+            context = RequestContext(request, {
+                'current_page': page,
+                'lang': language_id
+            })
+            with context.bind_template(_template.template):
+                content = placeholder.render(context)
+                return HttpResponse(content)
+    raise Http404
+
+
+@staff_member_required
 @csrf_exempt
 def delete_content(request, page_id, language_id):
     page = get_object_or_404(Page, pk=page_id)
-    perm = PagePermission(request.user).check('delete', page=page,
-            lang=language_id, method='POST')
+    perm = request.user.has_perm('pages.delete_page')
     if not perm:
         raise Http404
 
     for c in Content.objects.filter(page=page, language=language_id):
         c.delete()
 
-    destination = request.REQUEST.get('next', request.META.get('HTTP_REFERER',
-        '/admin/pages/page/%s/' % page_id))
+    destination = request.POST.get(
+        'next',
+        request.META.get(
+            'HTTP_REFERER',
+            reverse("admin:pages_page_change", args=[page_id])
+        )
+    )
     return HttpResponseRedirect(destination)
-delete_content = staff_member_required(delete_content)
 
+
+@staff_member_required
 def traduction(request, page_id, language_id):
     """Traduction helper."""
     page = Page.objects.get(pk=page_id)
@@ -93,20 +169,22 @@ def traduction(request, page_id, language_id):
         Content.objects.get_content(page, language_id, "title")
         is None
     )
-    return render_to_response('pages/traduction_helper.html', {
-        'page':page,
-        'lang':lang,
-        'language_error':language_error,
-        'placeholders':placeholders,
-    }, context_instance=RequestContext(request))
-traduction = staff_member_required(traduction)
+    return render(request, 'pages/traduction_helper.html', {
+        'page': page,
+        'lang': lang,
+        'language_error': language_error,
+        'placeholders': placeholders,
+    })
 
+
+@staff_member_required
 def get_content(request, page_id, content_id):
     """Get the content for a particular page"""
     content = Content.objects.get(pk=content_id)
     return HttpResponse(content.body)
-get_content = staff_member_required(get_content)
 
+
+@staff_member_required
 @csrf_exempt
 def move_page(request, page_id, extra_context=None):
     """Move the page to the requested target, at the given
@@ -134,17 +212,24 @@ def move_page(request, page_id, extra_context=None):
             return list_pages_ajax(request, invalid_move)
     return HttpResponseRedirect('../../')
 
+
+@staff_member_required
 def sub_menu(request, page_id):
     """Render the children of the requested page with the sub_menu
     template."""
     page = Page.objects.get(id=page_id)
     pages = page.children.all()
     page_languages = settings.PAGE_LANGUAGES
-    perms = PagePermission(request.user)
-    return render_to_response("admin/pages/page/sub_menu.html", {
-        'can_publish': perms.check('publish'),
-        'page':page,
-        'pages':pages,
-        'page_languages':page_languages,
-    },        context_instance=RequestContext(request))
-sub_menu = staff_member_required(sub_menu)
+    return render(request, "admin/pages/page/sub_menu.html", {
+        'can_publish': request.user.has_perm('pages.can_publish'),
+        'page': page,
+        'pages': pages,
+        'page_languages': page_languages,
+    })
+
+import os
+@staff_member_required
+def get_media_url(request, media_id):
+    """Get media URL."""
+    media = Media.objects.get(id=media_id)
+    return HttpResponse(media.url.name)
