@@ -5,18 +5,34 @@ from pages.cache import cache
 from pages.utils import normalize_url, get_now
 from pages.phttp import get_slug
 
-from django.db import models, connection
+from django.db import models
 from django.db.models import Q
-from django.core.exceptions import ObjectDoesNotExist
-from django.contrib.auth.models import User
-from django.db.models import Avg, Max, Min, Count
-from django.contrib.sites.models import Site
+from django.db.models import Max
 from django.conf import settings as global_settings
-from django.utils.translation import ugettext_lazy as _
 
 from mptt.managers import TreeManager
 
 
+class FakePage():
+    """
+    Used for pageless Content object
+    """
+
+    def __init__(self):
+        self.id = -999  # an impossible number to get with DB id, do not change
+        self._content_dict = {}
+        self.freeze_date = None
+
+    def invalidate(self, content_type):
+        key = ContentManager.PAGE_CONTENT_DICT_KEY % (
+            self.id, content_type, 0)
+        cache.delete(key)
+        self._content_dict = None
+
+
+fake_page = FakePage()
+
+
 class PageManager(TreeManager):
     """
     Page manager provide several filters to obtain pages :class:`QuerySet`
@@ -29,24 +45,6 @@ class PageManager(TreeManager):
             return super(PageManager, self).get_query_set().filter(
                 sites=global_settings.SITE_ID)
 
-    def populate_pages(self, parent=None, child=5, depth=5):
-        """Create a population of :class:`Page <pages.models.Page>`
-        for testing purpose."""
-        from pages.models import Content
-        author = User.objects.all()[0]
-        if depth == 0:
-            return
-        p = self.model(parent=parent, author=author,
-            status=self.model.PUBLISHED)
-        p.save()
-        p = self.get(id=p.id)
-        Content(body='page-' + str(p.id), type='title',
-            language=settings.PAGE_DEFAULT_LANGUAGE, page=p).save()
-        Content(body='page-' + str(p.id), type='slug',
-            language=settings.PAGE_DEFAULT_LANGUAGE, page=p).save()
-        for child in range(1, child + 1):
-            self.populate_pages(parent=p, child=child, depth=(depth - 1))
-
     def on_site(self, site_id=None):
         """Return a :class:`QuerySet` of pages that are published on the site
         defined by the ``SITE_ID`` setting.
@@ -66,7 +64,7 @@ class PageManager(TreeManager):
     def navigation(self):
         """Creates a :class:`QuerySet` of the published root pages."""
         return self.on_site().filter(
-                status=self.model.PUBLISHED).filter(parent__isnull=True)
+            status=self.model.PUBLISHED).filter(parent__isnull=True)
 
     def hidden(self):
         """Creates a :class:`QuerySet` of the hidden pages."""
@@ -131,8 +129,8 @@ class PageManager(TreeManager):
             pages_list = pages_list.exclude(status=self.model.DRAFT)
         if len(pages_list) == 1:
             if(settings.PAGE_USE_STRICT_URL and
-                pages_list[0].get_complete_slug(lang) != complete_path):
-                    return None
+                    pages_list[0].get_complete_slug(lang) != complete_path):
+                return None
             return pages_list[0]
         # if more than one page is matching the slug,
         # we need to use the full URL
@@ -141,7 +139,14 @@ class PageManager(TreeManager):
                 if page.get_complete_slug(lang) == complete_path:
                     return page
         return None
-        
+
+    def from_slug(self, slug):
+        from pages.models import Content
+        content = Content.objects.get_content_slug_by_slug(slug)
+        if content is None:
+            raise ValueError("Slug '%s' didn't match any content." % slug)
+        return content.page
+
 
 class ContentManager(models.Manager):
     """:class:`Content <pages.models.Content>` manager methods"""
@@ -178,20 +183,23 @@ class ContentManager(models.Manager):
         :param body: the content of the Content object.
         """
         try:
-            content = self.filter(page=page, language=language,
-                                  type=ctype).latest('creation_date')
+            content = self.filter(
+                page=page, language=language,
+                type=ctype).latest('creation_date')
             if content.body == body:
                 return content
         except self.model.DoesNotExist:
             pass
-        content = self.create(page=page, language=language, body=body,
-                type=ctype)
+        content = self.create(
+            page=page, language=language, body=body,
+            type=ctype)
 
         # Delete old revisions
         if settings.PAGE_CONTENT_REVISION_DEPTH:
-            oldest_content = self.filter(page=page, language=language,
-                type=ctype).order_by('-creation_date'
-                )[settings.PAGE_CONTENT_REVISION_DEPTH:]
+            oldest_content = self.filter(
+                page=page, language=language,
+                type=ctype
+            ).order_by('-creation_date')[settings.PAGE_CONTENT_REVISION_DEPTH:]
             for c in oldest_content:
                 c.delete()
 
@@ -203,7 +211,7 @@ class ContentManager(models.Manager):
         params = {
             'language': language,
             'type': ctype,
-            'page': page
+            'page': None if page is fake_page else page
         }
         if page.freeze_date:
             params['creation_date__lte'] = page.freeze_date
@@ -218,6 +226,11 @@ class ContentManager(models.Manager):
         :param ctype: the content type.
         :param language_fallback: fallback to another language if ``True``.
         """
+        if page is None:
+            page = fake_page
+
+        if " " in ctype:
+            raise ValueError("Ctype cannot contain spaces.")
         if not language:
             language = settings.PAGE_DEFAULT_LANGUAGE
 
