@@ -5,27 +5,29 @@ from pages.utils import get_placeholders, normalize_url, get_now
 from pages.managers import PageManager, ContentManager
 from pages.managers import PageAliasManager
 from pages import settings
+from pages.utils import slugify
+# checks
+from pages import checks
 
-from datetime import datetime
 from django.db import models
 from django.conf import settings as django_settings
-from django.contrib.auth.models import User, SiteProfileNotAvailable
-from django.db.models import Max
 from django.utils.translation import ugettext_lazy as _
 from django.utils.safestring import mark_safe
-from django.core.urlresolvers import reverse
-from django.core.exceptions import ObjectDoesNotExist
-from django.contrib.sites.models import Site
+from django.urls import reverse
 from django.conf import settings as global_settings
-from django.utils.encoding import python_2_unicode_compatible
 
 
 from mptt.models import MPTTModel
+from mptt.fields import TreeForeignKey
+import uuid
+import os
 
 PAGE_CONTENT_DICT_KEY = ContentManager.PAGE_CONTENT_DICT_KEY
 
+if settings.PAGE_USE_SITE_ID:
+    from django.contrib.sites.models import Site
+
 
-@python_2_unicode_compatible
 class Page(MPTTModel):
     """
     This model contain the status, dates, author, template.
@@ -70,10 +72,14 @@ class Page(MPTTModel):
     CHILDREN_KEY = 'children_%d'
     PUB_CHILDREN_KEY = 'pub_children_%d'
 
-    author = models.ForeignKey(django_settings.AUTH_USER_MODEL, verbose_name=_('author'))
+    # used to identify pages across different databases
+    uuid = models.UUIDField(default=uuid.uuid4, editable=False)
 
-    parent = models.ForeignKey('self', null=True, blank=True,
-            related_name='children', verbose_name=_('parent'))
+    author = models.ForeignKey(django_settings.AUTH_USER_MODEL,
+            verbose_name=_('author'), on_delete=models.CASCADE)
+
+    parent = TreeForeignKey('self', null=True, blank=True,
+            related_name='children', verbose_name=_('parent'), on_delete=models.CASCADE)
     creation_date = models.DateTimeField(_('creation date'), editable=False,
             default=get_now)
     publication_date = models.DateTimeField(_('publication date'),
@@ -98,14 +104,14 @@ class Page(MPTTModel):
 
     if settings.PAGE_USE_SITE_ID:
         sites = models.ManyToManyField(Site,
-                default=lambda: [global_settings.SITE_ID],
-                help_text=_('The site(s) the page is accessible at.'),
-                verbose_name=_('sites'))
+            default=[global_settings.SITE_ID],
+            help_text=_('The site(s) the page is accessible at.'),
+            verbose_name=_('sites'))
 
-    redirect_to_url = models.CharField(max_length=200, null=True, blank=True)
+    redirect_to_url = models.CharField(_('redirect to url'), max_length=200, null=True, blank=True)
 
     redirect_to = models.ForeignKey('self', null=True, blank=True,
-            related_name='redirected_pages')
+        related_name='redirected_pages', verbose_name=_('redirect to'), on_delete=models.SET_NULL)
 
     # Managers
     objects = PageManager()
@@ -174,20 +180,20 @@ class Page(MPTTModel):
 
     def get_children(self):
         """Cache superclass result"""
-        key = self.CHILDREN_KEY % self.id
-        children = cache.get(key, None)
-        if children is None:
-            children = super(Page, self).get_children()
-            cache.set(key, children)
+        key = self.CHILDREN_KEY % self.pk
+        #children = cache.get(key, None)
+        # if children is None:
+        children = super(Page, self).get_children()
+        #cache.set(key, children)
         return children
 
     def published_children(self):
         """Return a :class:`QuerySet` of published children page"""
-        key = self.PUB_CHILDREN_KEY % self.id
-        children = cache.get(key, None)
-        if children is None:
-            children = Page.objects.filter_published(self.get_children())
-            cache.set(key, children)
+        key = self.PUB_CHILDREN_KEY % self.pk
+        #children = cache.get(key, None)
+        # if children is None:
+        children = Page.objects.filter_published(self.get_children()).all()
+        #cache.set(key, children)
         return children
 
     def get_children_for_frontend(self):
@@ -202,7 +208,7 @@ class Page(MPTTModel):
     def move_to(self, target, position='first-child'):
         """Invalidate cache when moving"""
 
-        # Invalidate both in case position matters, 
+        # Invalidate both in case position matters,
         # otherwise only target is needed.
         self.invalidate()
         target.invalidate()
@@ -211,10 +217,10 @@ class Page(MPTTModel):
     def invalidate(self):
         """Invalidate cached data for this page."""
 
-        cache.delete(self.PAGE_LANGUAGES_KEY % (self.id))
+        cache.delete(self.PAGE_LANGUAGES_KEY % (self.pk))
         cache.delete('PAGE_FIRST_ROOT_ID')
-        cache.delete(self.CHILDREN_KEY % self.id)
-        cache.delete(self.PUB_CHILDREN_KEY % self.id)
+        cache.delete(self.CHILDREN_KEY % self.pk)
+        cache.delete(self.PUB_CHILDREN_KEY % self.pk)
         # XXX: Should this have a depth limit?
         if self.parent_id:
             self.parent.invalidate()
@@ -222,21 +228,31 @@ class Page(MPTTModel):
         self._complete_slug = None
         self._content_dict = dict()
 
-        p_names = [p.ctype for p in get_placeholders(self.get_template())]
+        placeholders = get_placeholders(self.get_template())
+
+        p_names = [p.ctype for p in placeholders]
         if 'slug' not in p_names:
             p_names.append('slug')
         if 'title' not in p_names:
             p_names.append('title')
+
+        from pages.managers import fake_page
+        shared = [p for p in placeholders if p.shared]
+        for share in shared:
+            fake_page.invalidate(share.ctype)
+
         # delete content cache, frozen or not
         for name in p_names:
             # frozen
-            cache.delete(PAGE_CONTENT_DICT_KEY %
-                (self.id, name, 1))
+            cache.delete(
+                PAGE_CONTENT_DICT_KEY %
+                (self.pk, name, 1))
             # not frozen
-            cache.delete(PAGE_CONTENT_DICT_KEY %
-                (self.id, name, 0))
+            cache.delete(
+                PAGE_CONTENT_DICT_KEY %
+                (self.pk, name, 0))
 
-        cache.delete(self.PAGE_URL_KEY % (self.id))
+        cache.delete(self.PAGE_URL_KEY % (self.pk))
 
     def get_languages(self):
         """
@@ -244,23 +260,22 @@ class Page(MPTTModel):
         """
         if self._languages:
             return self._languages
-        self._languages = cache.get(self.PAGE_LANGUAGES_KEY % (self.id))
+        self._languages = cache.get(self.PAGE_LANGUAGES_KEY % (self.pk))
         if self._languages is not None:
             return self._languages
 
         languages = [c['language'] for
-                            c in Content.objects.filter(page=self,
-                            type="slug").values('language')]
+            c in Content.objects.filter(page=self,
+            type="slug").values('language')]
         # remove duplicates
-        languages = list(set(languages))
-        languages.sort()
-        cache.set(self.PAGE_LANGUAGES_KEY % (self.id), languages)
+        languages = sorted(set(languages))
+        cache.set(self.PAGE_LANGUAGES_KEY % (self.pk), languages)
         self._languages = languages
         return languages
 
     def is_first_root(self):
         """Return ``True`` if this page is the first root pages."""
-        parent_cache_key = 'PARENT_FOR_%d' % self.id
+        parent_cache_key = 'PARENT_FOR_%d' % self.pk
         has_parent = cache.get(parent_cache_key, None)
         if has_parent is None:
             has_parent = not not self.parent
@@ -272,7 +287,7 @@ class Page(MPTTModel):
             return self._is_first_root
         first_root_id = cache.get('PAGE_FIRST_ROOT_ID')
         if first_root_id is not None:
-            self._is_first_root = first_root_id == self.id
+            self._is_first_root = first_root_id == self.pk
             return self._is_first_root
         try:
             first_root_id = Page.objects.root().values('id')[0]['id']
@@ -280,7 +295,7 @@ class Page(MPTTModel):
             first_root_id = None
         if first_root_id is not None:
             cache.set('PAGE_FIRST_ROOT_ID', first_root_id)
-        self._is_first_root = self.id == first_root_id
+        self._is_first_root = self.pk == first_root_id
         return self._is_first_root
 
     def get_template(self):
@@ -322,13 +337,13 @@ class Page(MPTTModel):
 
         :param perms: the level of permission of the concerned user.
         """
-        exclude_list = [self.id]
+        exclude_list = [self.pk]
         for p in self.get_descendants():
             exclude_list.append(p.id)
         return Page.objects.exclude(id__in=exclude_list)
-    
-    ### Content methods
-    
+
+    # Content methods
+
     def get_content(self, language, ctype, language_fallback=False):
         """Shortcut method for retrieving a piece of page content
 
@@ -371,9 +386,9 @@ class Page(MPTTModel):
             except Content.DoesNotExist:
                 pass
         return content_list
-        
+
     ### Title and slug
-    
+
     def get_url_path(self, language=None):
         """Return the URL's path component. Add the language prefix if
         ``PAGE_USE_LANGUAGE_PREFIX`` setting is set to ``True``.
@@ -398,7 +413,7 @@ class Page(MPTTModel):
 
     def get_absolute_url(self, language=None):
         """Alias for `get_url_path`.
-        
+
         :param language: the wanted url language.
         """
         return self.get_url_path(language=language)
@@ -414,7 +429,7 @@ class Page(MPTTModel):
         if self._complete_slug and language in self._complete_slug:
             return self._complete_slug[language]
 
-        self._complete_slug = cache.get(self.PAGE_URL_KEY % (self.id))
+        self._complete_slug = cache.get(self.PAGE_URL_KEY % (self.pk))
         if self._complete_slug is None:
             self._complete_slug = {}
         elif language in self._complete_slug:
@@ -425,7 +440,7 @@ class Page(MPTTModel):
         else:
             url = '%s' % self.slug(language)
 
-        key = self.ANCESTORS_KEY % self.id
+        key = self.ANCESTORS_KEY % self.pk
         ancestors = cache.get(key, None)
         if ancestors is None:
             ancestors = self.get_ancestors(ascending=True)
@@ -435,7 +450,7 @@ class Page(MPTTModel):
             url = ancestor.slug(language) + '/' + url
 
         self._complete_slug[language] = url
-        cache.set(self.PAGE_URL_KEY % (self.id), self._complete_slug)
+        cache.set(self.PAGE_URL_KEY % (self.pk), self._complete_slug)
         return url
 
     def slug_with_level(self, language=None):
@@ -457,6 +472,8 @@ class Page(MPTTModel):
         """
 
         slug = self.get_content(language, 'slug', language_fallback=fallback)
+        if slug == '':
+            return "Page {0}".format(self.pk)
 
         return slug
 
@@ -472,8 +489,8 @@ class Page(MPTTModel):
             language = settings.PAGE_DEFAULT_LANGUAGE
 
         return self.get_content(language, 'title', language_fallback=fallback)
-        
-    ### Formating methods
+
+    # Formating methods
 
     def margin_level(self):
         """Used in the admin menu to create the left margin."""
@@ -481,28 +498,27 @@ class Page(MPTTModel):
 
     def __str__(self):
         """Representation of the page, saved or not."""
-        if self.id:
+        if self.pk:
             # without ID a slug cannot be retrieved
-            slug = self.slug()
-            if slug:
-                return slug
-            return "Page {0}".format(self.id)
+            return self.slug()
         return "Page without id"
 
 
-@python_2_unicode_compatible
 class Content(models.Model):
     """A block of content, tied to a :class:`Page <pages.models.Page>`,
     for a particular language"""
 
-    # languages could have five characters : Brazilian Portuguese is pt-br
-    language = models.CharField(_('language'), max_length=5, blank=False)
-    body = models.TextField(_('body'))
-    type = models.CharField(_('type'), max_length=100, blank=False,
+    # languages could have seven characters : Simplified Chinese is zh-hans
+    language = models.CharField(_('language'), max_length=7, blank=False)
+    body = models.TextField(_('body'), blank=True)
+    type = models.CharField(
+        _('type'), max_length=100, blank=False,
         db_index=True)
-    page = models.ForeignKey(Page, verbose_name=_('page'))
+    page = models.ForeignKey(Page, verbose_name=_('page'), null=True,
+        on_delete=models.CASCADE)
 
-    creation_date = models.DateTimeField(_('creation date'), editable=False,
+    creation_date = models.DateTimeField(
+        _('creation date'), editable=False,
         default=get_now)
     objects = ContentManager()
 
@@ -512,13 +528,13 @@ class Content(models.Model):
         verbose_name_plural = _('contents')
 
     def __str__(self):
-        return "{0} :: {1}".format(self.page.slug(), self.body[0:15])
+        return u"{0} :: {1}".format(self.page.slug(), self.body[0:15])
+
 
-@python_2_unicode_compatible
 class PageAlias(models.Model):
     """URL alias for a :class:`Page <pages.models.Page>`"""
     page = models.ForeignKey(Page, null=True, blank=True,
-        verbose_name=_('page'))
+        verbose_name=_('page'), on_delete=models.CASCADE)
     url = models.CharField(max_length=255, unique=True)
     objects = PageAliasManager()
 
@@ -533,3 +549,58 @@ class PageAlias(models.Model):
     def __str__(self):
         return "{0} :: {1}".format(self.url, self.page.get_complete_slug())
 
+
+def media_filename(instance, filename):
+    avoid_collision = uuid.uuid4().hex[:8]
+    name_parts = filename.split('.')
+    if len(name_parts) > 1:
+        name = slugify('.'.join(name_parts[:-1]), allow_unicode=True)
+        ext = slugify(name_parts[-1])
+        name = name + '.' + ext
+    else:
+        name = slugify(filename)
+    filename = os.path.join(
+        settings.PAGE_UPLOAD_ROOT,
+        'medias',
+        name
+    )
+    return filename
+
+
+class Media(models.Model):
+    """Media model :class:`Media <pages.models.Media>`"""
+    title = models.CharField(max_length=255, blank=True)
+    description = models.TextField(blank=True)
+    url = models.FileField(upload_to=media_filename)
+    extension = models.CharField(max_length=32, blank=True, editable=False)
+    creation_date = models.DateTimeField(_('creation date'), editable=False,
+            default=get_now)
+
+    def image(self):
+        if self.extension in ['png', 'jpg', 'jpeg']:
+            return mark_safe('<img width="60" src="%s" />' % os.path.join(
+                settings.PAGES_MEDIA_URL, self.url.name))
+        if self.extension == 'pdf':
+            return mark_safe('<i class="fa fa-file-pdf-o" aria-hidden="true"></i>')
+        if self.extension in ['doc', 'docx']:
+            return mark_safe('<i class="fa fa-file-word-o" aria-hidden="true"></i>')
+        if self.extension in ['zip', 'gzip', 'rar']:
+            return mark_safe('<i class="fa fa-file-archive-o" aria-hidden="true"></i>')
+        return mark_safe('<i class="fa fa-file-o" aria-hidden="true"></i>')
+    image.short_description = _('Thumbnail')
+
+    class Meta:
+        verbose_name_plural = _('Medias')
+
+    def save(self, *args, **kwargs):
+        parts = self.url.name.split('.')
+        if len(parts) > 1:
+            self.extension = parts[-1].lower()
+        if not self.title:
+            parts = self.url.name.split('/')
+            self.title = parts[-1]
+
+        super(Media, self).save(*args, **kwargs)
+
+    def __str__(self):
+        return self.url.name
