@@ -1,6 +1,8 @@
-from django.db.models import Avg, Max, Min, Count
-from django.contrib.auth.models import User, SiteProfileNotAvailable
-from django.core.exceptions import ObjectDoesNotExist
+from django.db.models import Max
+from django.utils.translation import ugettext_lazy as _
+from django.contrib.sites.models import Site
+from django.conf import settings as global_settings
+from django.contrib.auth import get_user_model
 
 from pages.models import Page, Content
 from pages.managers import PageManager
@@ -10,7 +12,7 @@ from pages import settings
 from datetime import datetime
 import json as _json
 
-ISODATE_FORMAT = '%Y-%m-%dT%H:%M:%S.%f' # for parsing dates from JSON
+ISODATE_FORMAT = '%Y-%m-%dT%H:%M:%S.%f'  # for parsing dates from JSON
 JSON_PAGE_EXPORT_NAME = 'gerbi_cms_page_export_version'
 JSON_PAGE_EXPORT_VERSION = 4
 # make it readable -- there are better ways to save space
@@ -23,12 +25,12 @@ def monkeypatch_remove_pages_site_restrictions():
     removing customized get_query_set. Only actually matters
     if PAGE_HIDE_SITES is set
     """
-    from pages.managers import PageManager
     try:
         del PageManager.get_query_set
     except AttributeError:
         pass
 
+
 def dump_json_data(page):
     """
     Return a python dict representation of this page for use as part of
@@ -59,7 +61,7 @@ def dump_json_data(page):
         out = {}
         for p in get_placeholders(page.get_template()):
             if p.ctype in ('title', 'slug'):
-                continue # these were already included
+                continue  # these were already included
             out[p.name] = language_content(p.name)
         return out
 
@@ -68,12 +70,11 @@ def dump_json_data(page):
 
     def custom_email(user):
         """Allow a user's profile to return an email for the user."""
-        try:
-            profile = user.get_profile()
-        except (SiteProfileNotAvailable, ObjectDoesNotExist):
-            return user.email
-        get_email = getattr(profile, 'get_email', None)
-        return get_email() if get_email else user.email
+        return user.email
+
+    tags = []
+    if settings.PAGE_TAGGING:
+        tags = [tag.name for tag in page.tags.all()]
 
     return {
         'complete_slug': dict(
@@ -101,8 +102,10 @@ def dump_json_data(page):
             ) if page.redirect_to is not None else None,
         'content': placeholder_content(),
         'content_language_updated_order': languages,
+        'tags': tags,
     }
 
+
 def update_redirect_to_from_json(page, redirect_to_complete_slugs):
     """
     The second pass of create_and_update_from_json_data
@@ -123,8 +126,8 @@ def update_redirect_to_from_json(page, redirect_to_complete_slugs):
         messages.append(_("Could not find page for redirect-to field"
             " '%s'") % (s,))
     return messages
-    
-    
+
+
 def create_and_update_from_json_data(d, user):
     """
     Create or update page based on python dict d loaded from JSON data.
@@ -168,26 +171,17 @@ def create_and_update_from_json_data(d, user):
         page = Page(parent=parent)
         created = True
 
+    user_model = get_user_model()
+
     def custom_get_user_by_email(email):
         """
-        Allow the user profile class to look up a user by email
-        address
+        Simplified version
         """
-        # bit of an unpleasant hack that requres the logged-in
-        # user has a profile, but I don't want to reproduce the
-        # code in get_profile() here
-        try:
-            profile = user.get_profile()
-        except (SiteProfileNotAvailable, ObjectDoesNotExist):
-            return User.objects.get(email=email)
-        get_user_by_email = getattr(profile, 'get_user_by_email', None)
-        if get_user_by_email:
-            return get_user_by_email(email)
-        return User.objects.get(email=email)
+        return user_model.objects.get(email=email)
 
     try:
         page.author = custom_get_user_by_email(d['author_email'])
-    except (User.DoesNotExist, User.MultipleObjectsReturned):
+    except (user_model.DoesNotExist, user_model.MultipleObjectsReturned):
         page.author = user
         messages.append(_("Original author '%s' not found")
             % (d['author_email'],))
@@ -210,6 +204,17 @@ def create_and_update_from_json_data(d, user):
 
     page.save()
 
+    # Add tags
+    if settings.PAGE_TAGGING:
+        from taggit.models import Tag
+        tags = d.get('tags', [])
+        page.tags.clear()
+        if tags:
+            for tag in tags:
+                Tag.objects.get_or_create(name=tag)
+                page.tags.add(tag)
+            page.save()
+
     if settings.PAGE_USE_SITE_ID:
         if d['sites']:
             for site in d['sites']:
@@ -235,8 +240,8 @@ def create_and_update_from_json_data(d, user):
             create_content(lang, ctype, langs_bodies[lang])
 
     return page, created, messages
-    
-    
+
+
 def pages_to_json(queryset):
     """
     Return a JSON string export of the pages in queryset.
@@ -248,6 +253,7 @@ def pages_to_json(queryset):
             'pages': [dump_json_data(page) for page in queryset]},
         indent=JSON_PAGE_EXPORT_INDENT, sort_keys=True)
 
+
 def json_to_pages(json, user, preferred_lang=None):
     """
     Attept to create/update pages from JSON string json.  user is the
@@ -320,7 +326,7 @@ def validate_pages_json_data(d, preferred_lang):
                 continue
             seen_complete_slugs[lang].add(s)
 
-            if '/' not in s: # root level, no parent req'd
+            if '/' not in s:  # root level, no parent req'd
                 seen_parent = True
             if not seen_parent:
                 parent_slug, ignore = s.rsplit('/', 1)
@@ -356,125 +362,3 @@ def validate_pages_json_data(d, preferred_lang):
             continue
 
     return errors
-
-
-def pages_to_json(queryset):
-    """
-    Return a JSON string export of the pages in queryset.
-    """
-    # selection may be in the wrong order, and order matters
-    queryset = queryset.order_by('tree_id', 'lft')
-    return _json.dumps(
-        {JSON_PAGE_EXPORT_NAME: JSON_PAGE_EXPORT_VERSION,
-            'pages': [dump_json_data(page) for page in queryset]},
-        indent=JSON_PAGE_EXPORT_INDENT, sort_keys=True)
-
-
-def json_to_pages(json, user, preferred_lang=None):
-    """
-    Attept to create/update pages from JSON string json.  user is the
-    user that will be used when creating a page if a page's original
-    author can't be found.  preferred_lang is the language code of the
-    slugs to include in error messages (defaults to
-    settings.PAGE_DEFAULT_LANGUAGE).
-
-    Returns (errors, pages_created) where errors is a list of strings
-    and pages_created is a list of: (page object, created bool,
-    messages list of strings) tuples.
-
-    If any errors are detected there the error list will contain
-    information for the user and no pages will be created/updated.
-    """
-    from pages.models import Page
-    if not preferred_lang:
-        preferred_lang = settings.PAGE_DEFAULT_LANGUAGE
-
-    d = _json.loads(json)
-    try:
-        errors = validate_pages_json_data(d, preferred_lang)
-    except KeyError as e:
-        errors = [_('JSON file is invalid: %s') % (e.args[0],)]
-
-    pages_created = []
-    if not errors:
-        # pass one
-        for p in d['pages']:
-            pages_created.append(
-                create_and_update_from_json_data(p, user))
-        # pass two
-        for p, results in zip(d['pages'], pages_created):
-            page, created, messages = results
-            rtcs = p['redirect_to_complete_slug']
-            if rtcs:
-                messages.extend(page.update_redirect_to_from_json(rtcs))
-        # clean up MPTT links
-        Page.objects.rebuild()
-
-    return errors, pages_created
-
-
-def validate_pages_json_data(d, preferred_lang):
-    """
-    Check if an import of d will succeed, and return errors.
-
-    errors is a list of strings.  The import should proceed only if errors
-    is empty.
-    """
-    from pages.models import Page
-    errors = []
-
-    seen_complete_slugs = dict(
-        (lang[0], set()) for lang in settings.PAGE_LANGUAGES)
-
-    valid_templates = set(t[0] for t in settings.get_page_templates())
-    valid_templates.add(settings.PAGE_DEFAULT_TEMPLATE)
-
-    if d[JSON_PAGE_EXPORT_NAME] != JSON_PAGE_EXPORT_VERSION:
-        return [_('Unsupported file version: %s') % repr(
-            d[JSON_PAGE_EXPORT_NAME])], []
-    pages = d['pages']
-    for p in pages:
-        # use the complete slug as a way to identify pages in errors
-        slug = p['complete_slug'].get(preferred_lang, None)
-        seen_parent = False
-        for lang, s in list(p['complete_slug'].items()):
-            if lang not in seen_complete_slugs:
-                continue
-            seen_complete_slugs[lang].add(s)
-
-            if '/' not in s: # root level, no parent req'd
-                seen_parent = True
-            if not seen_parent:
-                parent_slug, ignore = s.rsplit('/', 1)
-                if parent_slug in seen_complete_slugs[lang]:
-                    seen_parent = True
-                else:
-                    parent = Page.objects.from_path(parent_slug, lang,
-                        exclude_drafts=False)
-                    if parent and parent.get_complete_slug(lang) == parent_slug:
-                        # parent not included, but exists on site
-                        seen_parent = True
-            if not slug:
-                slug = s
-
-        if not slug:
-            errors.append(_("%s has no common language with this site")
-                % (list(p['complete_slug'].values())[0],))
-            continue
-
-        if not seen_parent:
-            errors.append(_("%s did not include its parent page and a matching"
-                " one was not found on this site") % (slug,))
-
-        if p['template'] not in valid_templates:
-            errors.append(_("%s uses a template not found on this site: %s")
-                % (slug, p['template']))
-            continue
-
-        if set(p.ctype for p in get_placeholders(p['template']) if
-                p.ctype not in ('title', 'slug')) != set(p['content'].keys()):
-            errors.append(_("%s template contents are different than our "
-                "template: %s") % (slug, p['template']))
-            continue
-
-    return errors
\ No newline at end of file
