@@ -50,6 +50,8 @@ PAGE_TEMPLATES = get_setting('PAGE_TEMPLATES',
 
 def get_page_templates():
     """The callable that is used by the CMS."""
+    PAGE_TEMPLATES = get_setting('PAGE_TEMPLATES',
+        default_value=())
     if isinstance(PAGE_TEMPLATES, collections.Callable):
         return PAGE_TEMPLATES()
     else:
@@ -95,8 +97,8 @@ PAGE_CONTENT_REVISION_DEPTH = getattr(settings,
 #
 # PAGE_LANGUAGES = (
 #    ('zh-cn', gettext_noop('Chinese Simplified')),
-#    ('fr-ch', gettext_noop('Swiss french')),
-#    ('en-us', gettext_noop('US English')),
+#    ('fr', gettext_noop('Français')),
+#    ('en', gettext_noop('US English')),
 #)
 
 PAGE_LANGUAGES = get_setting('PAGE_LANGUAGES', raise_error=True)
@@ -165,11 +167,17 @@ PAGE_CONTENT_REVISION_EXCLUDE_LIST = getattr(settings,
     'PAGE_CONTENT_REVISION_EXCLUDE_LIST', ()
 )
 
-# URL that handles pages media and uses <STATIC_URL>/pages by default.
 PAGES_MEDIA_URL = get_setting('PAGES_MEDIA_URL')
 if not PAGES_MEDIA_URL:
-    media_url = get_setting('STATIC_URL', 'MEDIA_URL', raise_error=True)
-    PAGES_MEDIA_URL = str(media_url) + 'pages/'
+    media_url = get_setting('PAGES_MEDIA_URL', 'MEDIA_URL', raise_error=True)
+    PAGES_MEDIA_URL = str(media_url)
+
+PAGES_STATIC_URL = get_setting('PAGES_STATIC_URL')
+if not PAGES_STATIC_URL:
+    static_url = get_setting('PAGES_STATIC_URL', 'STATIC_URL', raise_error=True)
+    static_url = static_url + 'pages/'
+    PAGES_STATIC_URL = str(static_url)
+
 
 # Hide the slug's of the first root page ie: ``/home/`` becomes ``/``
 PAGE_HIDE_ROOT_SLUG = getattr(settings, 'PAGE_HIDE_ROOT_SLUG', False)
@@ -186,11 +194,6 @@ PAGE_SHOW_START_DATE = getattr(settings, 'PAGE_SHOW_START_DATE', False)
 # your database and null any pages with ``publication_end_date`` set.
 PAGE_SHOW_END_DATE = getattr(settings, 'PAGE_SHOW_END_DATE', False)
 
-# The page link filter enable a output filter on you content links. The goal
-# is to transform special page class into real links at the last moment.
-# This ensure that even if you have moved a page, the URL will remain correct.
-PAGE_LINK_FILTER = getattr(settings, 'PAGE_LINK_FILTER', False)
-
 # This setting is a function that can be defined if you need to pass extra
 # context dict to the pages templates. You can customize the way the function
 # is called by subclassing ``pages.views.Details``.
@@ -213,7 +216,12 @@ PAGE_ENABLE_TESTS = getattr(settings, 'PAGE_ENABLE_TESTS', False)
 PAGE_IMPORT_ENABLED = getattr(settings, 'PAGE_IMPORT_ENABLED', False)
 PAGE_EXPORT_ENABLED = getattr(settings, 'PAGE_EXPORT_ENABLED', False)
 
-# If you want to have a separate cache for cms, override this
-PAGE_CACHE_LOCATION = get_setting('PAGE_CACHE_LOCATION',
-    default_value='default')
+# Enable the API or not
+PAGE_API_ENABLED = getattr(settings, 'PAGE_API_ENABLED', False)
 
+# If you want to see screenshots from selenium tests override this
+PAGE_TESTS_SAVE_SCREENSHOTS = getattr(settings, 'PAGE_TESTS_SAVE_SCREENSHOTS',
+                                     False)
+
+# If you want to redirect to new page url
+PAGE_REDIRECT_OLD_SLUG = getattr(settings, 'PAGE_REDIRECT_OLD_SLUG', False)
