@@ -2,10 +2,16 @@ from django import template
 from django.forms import Textarea
 from django.utils import translation
 from django.conf import settings
+from django.contrib.admin.widgets import AdminTextareaWidget
 
 from pages.placeholders import PlaceholderNode
 from pages.placeholders import parse_placeholder
 
+try:
+    from ckeditor.widgets import CKEditorWidget
+except ImportError:
+    CKEditorWidget = AdminTextareaWidget
+
 register = template.Library()
 
 class CKEditorPlaceholderNode(PlaceholderNode):
@@ -13,8 +19,6 @@ class CKEditorPlaceholderNode(PlaceholderNode):
         if 'ckeditor' not in settings.INSTALLED_APPS:
             return fallback
 
-        from ckeditor.widgets import CKEditorWidget
-
         with_stmt = self.widget # name of the widget as called in template like...
         # {% ckeditor_placeholder "welcome" with text_wysiwym_widget:default%}
         splitted = with_stmt.split(":")
