@@ -6,7 +6,6 @@ PROJECT_DIR = os.path.dirname(__file__)
 TEST_PROJ = 'pages.testproj'
 
 DEBUG = True
-TEMPLATE_DEBUG = DEBUG
 USE_TZ = True
 
 ADMINS = (
@@ -20,14 +19,13 @@ MANAGERS = ADMINS
 DATABASES = {
     'default': {
         'ENGINE': 'django.db.backends.sqlite3',
-        'NAME': 'test.db'
+        'NAME': 'test.db',
+        'OPTIONS': {
+            'timeout': 10,
+        }
     }
 }
 
-# We still want to be ale to test with 1.1.X
-DATABASE_ENGINE = 'sqlite3'
-DATABASE_NAME = 'test.db'
-
 # Local time zone for this installation. Choices can be found here:
 # http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
 # although not all choices may be avilable on all operating systems.
@@ -59,30 +57,28 @@ FIXTURE_DIRS = [os.path.join(PROJECT_DIR, 'fixtures')]
 # Make this unique, and don't share it with anybody.
 SECRET_KEY = '*xq7m@)*f2awoj!spa0(jibsrz9%c0d=e(g)v*!17y(vx0ue_3'
 
-TEMPLATE_CONTEXT_PROCESSORS = (
+_TEMPLATE_CONTEXT_PROCESSORS = (
     "django.contrib.auth.context_processors.auth",
-    "django.core.context_processors.i18n",
-    "django.core.context_processors.debug",
-    "django.core.context_processors.request",
-    "django.core.context_processors.media",
+    "django.template.context_processors.i18n",
+    "django.template.context_processors.debug",
+    "django.template.context_processors.request",
+    "django.template.context_processors.media",
     "pages.context_processors.media",
-    #"staticfiles.context_processors.static_url",
 )
 
 INTERNAL_IPS = ('127.0.0.1',)
 
-MIDDLEWARE_CLASSES = (
+MIDDLEWARE = (
     'django.contrib.sessions.middleware.SessionMiddleware',
     'django.middleware.locale.LocaleMiddleware',
     'django.middleware.common.CommonMiddleware',
     'django.contrib.auth.middleware.AuthenticationMiddleware',
-    'django.middleware.doc.XViewMiddleware',
     'django.contrib.messages.middleware.MessageMiddleware'
 )
 
 ROOT_URLCONF = TEST_PROJ + '.urls'
 
-TEMPLATE_DIRS = (
+_TEMPLATE_DIRS = (
     # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
     # Always use forward slashes, even on Windows.
     # Don't forget to use absolute paths, not relative paths.
@@ -98,26 +94,20 @@ INSTALLED_APPS = (
     'django.contrib.admin',
     'django.contrib.sites',
     'django.contrib.sitemaps',
-    TEST_PROJ + '.documents',
-    #'tagging',
-    'pages',
-    'pages.plugins.jsonexport',
-    'pages.plugins.pofiles',
+    'rest_framework',
     'mptt',
-    #'staticfiles',
-    #'tinymce',
-    # disabled to make "setup.py test" to work properly
-    #'south',
-
+    'pages',
+    'taggit',
+    TEST_PROJ + '.documents',
+    'django.contrib.staticfiles',
     # these 2 package don't create any dependecies
-    'authority',
     # haystack change coverage score report by importing modules
     #'haystack',
 )
 
 # Default language code for this installation. All choices can be found here:
 # http://www.i18nguy.com/unicode/language-identifiers.html
-LANGUAGE_CODE = 'en-us'
+LANGUAGE_CODE = 'en'
 
 # This is defined here as a do-nothing function because we can't import
 # django.utils.translation -- that module depends on the settings.
@@ -126,8 +116,8 @@ gettext_noop = lambda s: s
 # languages you want to translate into the CMS.
 PAGE_LANGUAGES = (
     ('de', gettext_noop('German')),
-    ('fr-ch', gettext_noop('Swiss french')),
-    ('en-us', gettext_noop('US English')),
+    ('fr', gettext_noop('Français')),
+    ('en', gettext_noop('US English')),
 )
 
 # You should add here all language you want to accept as valid client
@@ -143,18 +133,31 @@ LANGUAGES = languages
 # be in the LANGUAGES config
 def language_mapping(lang):
     if lang.startswith('fr'):
-        # serve swiss french for everyone
-        return 'fr-ch'
+        # serve Français for everyone
+        return 'fr'
     return lang
 
 PAGE_LANGUAGE_MAPPING = language_mapping
 
 PAGE_DEFAULT_TEMPLATE = 'pages/examples/index.html'
 
+PAGE_API_ENABLED = True
+
+TEMPLATES = [
+    {
+        'BACKEND': 'django.template.backends.django.DjangoTemplates',
+        'APP_DIRS': True,
+        'DIRS': _TEMPLATE_DIRS,
+        'OPTIONS': {
+            'debug': DEBUG,
+            'context_processors': _TEMPLATE_CONTEXT_PROCESSORS,
+        },
+    },
+]
+
 PAGE_TEMPLATES = (
     ('pages/examples/nice.html', 'nice one'),
     ('pages/examples/cool.html', 'cool one'),
-    ('pages/examples/editor.html', 'raw editor'),
     ('pages/tests/untranslated.html', 'untranslated'),
 )
 
@@ -162,6 +165,8 @@ PAGE_SANITIZE_USER_INPUT = True
 
 PAGE_USE_SITE_ID = True
 
+PAGE_TAGGING = True
+
 HAYSTACK_SITECONF = 'example.search_sites'
 HAYSTACK_SEARCH_ENGINE = 'dummy'
 #HAYSTACK_WHOOSH_PATH = os.path.join(PROJECT_DIR, 'whoosh_index')
@@ -179,9 +184,11 @@ COVERAGE_HTML_REPORT = True
 COVERAGE_BRANCH_COVERAGE = False
 PAGE_ENABLE_TESTS = True
 
+PASSWORD_HASHERS = [
+    'django.contrib.auth.hashers.SHA1PasswordHasher'
+]
 
 try:
     from local_settings import *
 except ImportError:
     pass
-
