@@ -2,24 +2,20 @@
 """Django CMS come with a set of ready to use widgets that you can enable
 in the admin via a placeholder tag in your template."""
 
-from pages.settings import PAGES_MEDIA_URL, PAGE_TAGGING
+from pages.settings import PAGES_MEDIA_URL, PAGES_STATIC_URL
 from pages.settings import PAGE_LANGUAGES
 from pages.models import Page
 from pages.widgets_registry import register_widget
 
-from django.conf import settings
 from django import forms
-from django.forms import TextInput, Textarea, HiddenInput
-from django.forms import MultiWidget, FileInput
-from django.forms.util import flatatt
+from django.forms import TextInput, Textarea
+from django.forms import MultiWidget
+from django.forms import FileInput as DFileInput
 from django.contrib.admin.widgets import AdminTextInputWidget
 from django.contrib.admin.widgets import AdminTextareaWidget
 from django.utils.safestring import mark_safe
-from django.utils.html import format_html
-from django.utils.encoding import force_text
 from django.template.loader import render_to_string
-from django.core.exceptions import ObjectDoesNotExist
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext_lazy as _
 
 from os.path import join
 
@@ -32,12 +28,14 @@ register_widget(AdminTextareaWidget)
 class RichTextarea(Textarea):
     """A RichTextarea widget."""
     class Media:
-        js = [join(PAGES_MEDIA_URL, path) for path in (
+        js = [join(PAGES_STATIC_URL, path) for path in (
             'javascript/jquery.js',
+            'javascript/jquery.rte.js'
         )]
         css = {
-            'all': [join(PAGES_MEDIA_URL, path) for path in (
+            'all': [join(PAGES_STATIC_URL, path) for path in (
                 'css/rte.css',
+                'css/font-awesome.min.css'
             )]
         }
 
@@ -50,61 +48,98 @@ class RichTextarea(Textarea):
         rendered = super(RichTextarea, self).render(name, value, attrs)
         context = {
             'name': name,
+            'PAGES_STATIC_URL': PAGES_STATIC_URL,
             'PAGES_MEDIA_URL': PAGES_MEDIA_URL,
         }
         return rendered + mark_safe(render_to_string(
             'pages/widgets/richtextarea.html', context))
+
+
 register_widget(RichTextarea)
 
 
-class ImageInput(FileInput):
+insert_image_link = u'''
+<br>
+<button title='insert image from the media library' class='image-lookup-{name}'>
+    From media library
+</button>
+<input name="{name}-selected" id="{name}-selected" type="hidden">
+<span id="{name}-selected-value">
+</span>
 
-    def __init__(self, page=None, language=None, attrs=None, **kwargs):
-        self.language = language
-        self.page = page
-        super(ImageInput, self).__init__(attrs)
-
-    def render(self, name, value, attrs=None, **kwargs):
-        if not self.page:
-            field_content = _('Please save the page to show the image field')
-        else:
-            field_content = ''
-            if value:
-                field_content += _('Current file: %s<br/>') % value
-            field_content += super(ImageInput, self).render(name, attrs)
-            if value:
-                field_content += '''<br><label for="%s-delete">%s</label>
-                    <input name="%s-delete" id="%s-delete"
-                    type="checkbox" value="true">
-                    ''' % (name, _('Delete image'), name, name)
-        return mark_safe(field_content)
-register_widget(ImageInput)
+<br><label for="{name}-delete">
+<input name="{name}-delete" style="display:inline-block" id="{name}-delete" type="checkbox" value="true"> {del_msg}
+</label>
+<br style="clear:both">
 
 
-class FileInput(FileInput):
+<script>
+$(function(){{
+    function dismissRelatedLookupPopup(win, chosenId) {{
+        $.get('/admin/pages/page/' + chosenId + '/media-url/', function(response) {{
+            console.log(response);
+            $('#{name}-selected').val(response);
+            $('#{name}-selected-value').text(response);
+        }});
+        win.close();
+        window.dismissRelatedLookupPopup = oldDismissRelatedLookupPopup;
+        window.dismissAddRelatedObjectPopup = oldDismissAddRelatedObjectPopup;
+    }}
+    function showMediaAdminPopup() {{
+        var name = 'mediaWindowSelect';
+        var href = '/admin/pages/media/?_to_field=id&_popup=1';
+        window.dismissRelatedLookupPopup = dismissRelatedLookupPopup;
+        window.dismissAddRelatedObjectPopup = dismissRelatedLookupPopup;
+        var win = window.open(href, name, 'height=500,width=800,resizable=yes,scrollbars=yes');
+        win.focus();
+        return false;
+    }}
+    $('.image-lookup-{name}').click(function(e) {{
+        e.preventDefault();
+        showMediaAdminPopup();
+        return false;
+    }});
+}});
+
+</script>
+'''
+
+class FileInput(DFileInput):
 
     def __init__(self, page=None, language=None, attrs=None, **kwargs):
         self.language = language
         self.page = page
         super(FileInput, self).__init__(attrs)
 
+    please_save_msg = _('Please save the page to show the file field')
+    delete_msg = _('Delete file')
+
     def render(self, name, value, attrs=None, **kwargs):
         if not self.page:
-            field_content = _('Please save the page to show the file field')
+            field_content = self.please_save_msg
         else:
-            field_content = ''
+            field_content = '<span class="placeholder-fileinput">'
             if value:
                 field_content += _('Current file: %s<br/>') % value
+                field_content += '<hr>'
             field_content += super(FileInput, self).render(name, attrs)
-            if value:
-                field_content += '''<br><label for="%s-delete">%s</label>
-                    <input name="%s-delete" id="%s-delete"
-                    type="checkbox" value="true">
-                    ''' % (name, _('Delete file'), name, name)
+            field_content += insert_image_link.format(
+                name=name,
+                del_msg=self.delete_msg,
+                value=value)
+            field_content += '</span>'
         return mark_safe(field_content)
 register_widget(FileInput)
 
 
+class ImageInput(FileInput):
+
+    please_save_msg = _('Please save the page to show the image field')
+    delete_msg = _('Delete image')
+
+register_widget(ImageInput)
+
+
 class LanguageChoiceWidget(TextInput):
 
     def __init__(self, language=None, attrs=None, **kwargs):
@@ -127,16 +162,18 @@ class LanguageChoiceWidget(TextInput):
 
 class PageLinkWidget(MultiWidget):
     '''A page link `Widget` for the admin.'''
-    def __init__(self, attrs=None, page=None, language=None,
-        video_url=None, linkedpage=None, text=None):
-            l = [('', '----')]
-            for p in Page.objects.all():
-                l.append((p.id, str(p)))
-            widgets = [
-                forms.Select(choices=l),
-                TextInput(attrs=attrs)
-            ]
-            super(PageLinkWidget, self).__init__(widgets, attrs)
+
+    def __init__(
+            self, attrs=None, page=None, language=None,
+            video_url=None, linkedpage=None, text=None):
+        l = [('', '----')]
+        for p in Page.objects.all():
+            l.append((p.id, str(p)))
+        widgets = [
+            forms.Select(choices=l),
+            TextInput(attrs=attrs)
+        ]
+        super(PageLinkWidget, self).__init__(widgets, attrs)
 
     def decompress(self, value):
         import json
@@ -174,5 +211,3 @@ class PageLinkWidget(MultiWidget):
             <tr><td>text</td><td>%s</td></tr>
         </table>""" % tuple(rendered_widgets)
 register_widget(PageLinkWidget)
-
-
